package com.youli.mybatis.dao;

import java.sql.Connection;//1
import java.sql.DriverManager;//1
import java.sql.PreparedStatement;//1
import java.sql.ResultSet;//1
import java.sql.SQLException;
import java.util.ArrayList;//1
import java.util.List;//1

import org.apache.log4j.Logger;

import com.youli.mybatis.bean.Book;

/**
* author: youli
* 2022-9-22 8:16:44
*/
public class BookDao {
	
	private Logger insertLogger = Logger.getLogger("insert");
	private Logger updateLogger = Logger.getLogger("update");
	private Logger deleteLogger = Logger.getLogger("delete");
	private Logger selectLogger = Logger.getLogger("select");
	private Logger listLogger = Logger.getLogger("list");

	private String driver = "com.mysql.jdbc.Driver";//1
	private String url = "jdbc:mysql://localhost:3306/mybatis?useSSL=false";
	private String username = "root";
	private String password = "123456";
		
	/**
	 * 根据用户id获取1个用户对象
	 * @param id
	 * @return Book对象
	 * @throws ClassNotFoundException 
	 * @throws SQLException 
	 */
	public Book selectById(Integer bookId) throws Exception {
		Book book = null;
		//1、 加载数据库驱动
		Class.forName(driver); 
		//2、建立到数据库连接
		Connection conn = DriverManager.getConnection(url, username, password); //1
		//3、构造Sql语句，写入日志
		String sql = "select * from tbl_book where book_id = ?";
		selectLogger.debug(sql);
		//4、创建预编译对象
		PreparedStatement pst = conn.prepareStatement(sql);
		//5\参数赋值，写入日志
		pst.setInt(1, bookId);
		selectLogger.debug(pst);
		//6、执行sql操作，获取结果集
		ResultSet rs = pst.executeQuery();//1
		//7、遍历结果集
		while(rs.next()) {//1
			book = new Book();
			book.setBookId(rs.getInt("book_id"));
			book.setBookAuthor(rs.getString("book_author"));
			book.setBookName(rs.getString("book_name"));
			book.setPublisherName(rs.getString("publisher_name"));
		}
		//8、释放资源
		rs.close();
		pst.close();
		conn.close();
		//9、返回值
		return book;
	}
	
	/**
	 * 查询所有用户对象
	 * @return Book对象集合
	 * @throws Exception 
	 */
	public List<Book> selectList() throws Exception {
		List<Book> list = new ArrayList<>();
		//1、 加载数据库驱动
		Class.forName(driver); 
		//2、建立到数据库连接
		Connection conn = DriverManager.getConnection(url, username, password); 
		//3、构造Sql语句，写入日志
		String sql = "select * from tbl_book";//占位符
		listLogger.debug(sql);
		//4、创建预编译对象	，写入日志
		PreparedStatement pst = conn.prepareStatement(sql);
		listLogger.debug(pst);
		//5、执行sql操作，获取结果集
		ResultSet rs = pst.executeQuery();
		//6、遍历结果集
		while(rs.next()) {//1
			Book book = new Book();
			book.setBookId(rs.getInt("book_id"));
			book.setBookAuthor(rs.getString("book_author"));
			book.setBookName(rs.getString("book_name"));
			book.setPublisherName(rs.getString("publisher_name"));
			list.add(book);
		}
		//7、释放资源
		rs.close();
		pst.close();
		conn.close();
		//8返回值
		return list;
	}
	
	/**
	 * 新增1条记录
	 * @param book对象
	 * @return 整数，代表操作影响的行数
	 * @throws Exception 
	 */
	public int insert(Book book) throws Exception {
		int ret = -1;
		//1、 加载数据库驱动
		Class.forName(driver);
		//2、建立到数据库连接
		Connection conn = DriverManager.getConnection(url, username, password); //1
		//3、构造Sql语句，写入日志
		String sql = "insert into tbl_book(book_author,book_name,book_price,publisher_name)values(?,?,?,?)";
		insertLogger.debug(sql);
		//4、创建预编译对象
		PreparedStatement pst = conn.prepareStatement(sql);
		//5、参数赋值，写入日志
		pst.setString(1, book.getBookAuthor());
		pst.setString(2, book.getBookName());
		pst.setDouble(3, book.getBookPrice());
		pst.setString(4, book.getPublisherName());
		insertLogger.debug(pst);
		//6、执行sql操作，获取结果集
		ret = pst.executeUpdate();	
		//7、释放资源
		pst.close();
		conn.close();
		//8返回值
		return ret;
	}
	
	/**
	 *    根据id，修改条记录的名称
	 * @param book对象
	 * @return 整数，代表操作影响的行数
	 * @throws Exception 
	 */
	public int update(Book book) throws Exception {
		int ret = -1;
		//1、 加载数据库驱动,加载到虚拟机
		Class.forName(driver); //1
		//2、建立到数据库连接
		Connection conn = DriverManager.getConnection(url, username, password); //1
		//3、构造Sql语句，写入日志
		String sql = "update tbl_book set book_name = ? where book_id = ?";
		updateLogger.debug(sql);
		//4、预编译对象：创建SQL语句（命令）对象，增删改查，分为两类：增删改-executeUpdate方法；查询-executeQuery方法		
		PreparedStatement pst = conn.prepareStatement(sql);
		//5、参数赋值，写入日志
		pst.setString(1, book.getBookName());
		pst.setInt(2, book.getBookId());
		updateLogger.debug(pst);
		//6、执行sql操作，获取结果集
		ret = pst.executeUpdate();		
		//7、释放资源
		pst.close();
		conn.close();
		//8返回值
		return ret;
	}
	
	/**
	 * 	根据id删除1条记录
	 * @param id
	 * @return 整数，代表操作影响的行数
	 * @throws Exception 
	 */
	public int delete(Integer bookId) throws Exception {
		int ret = -1;
		//1、 加载数据库驱动,加载到虚拟机
		Class.forName(driver); //1
		//2、建立到数据库连接
		Connection conn = DriverManager.getConnection(url, username, password); //1
		//3、构造Sql语句，写入日志
		String sql = "delete from tbl_book where book_id = ?";
		deleteLogger.debug(sql);
		//4、创建预编译对象
		PreparedStatement pst = conn.prepareStatement(sql);
		//5、参数赋值，写入日志
		pst.setInt(1, bookId);
		deleteLogger.debug(pst);
		//6、执行sql操作，获取结果集
		ret = pst.executeUpdate();		
		//7、释放资源
		pst.close();
		conn.close();
		//8返回值
		return ret;
	}
}
